@@ -12,7 +12,7 @@ import requests
 import xlrd
 import yaml
 
-default_sheets = ['info', 'projects', 'config', 'scheduler']
+default_sheets = ['info', 'projects', 'config', 'scheduler', 'trigger']
 HEADERS = {'Content-Type': 'application/x-www-form-urlencoded', 'X-Requested-With': 'XMLHttpRequest'}
 
 
@@ -186,14 +186,107 @@ def make_zip(projects, base_dir):
 
 
 '''
-generate all flows file in base dirtory 
+you can choose either schedule  or trigger,not both of them 
 '''
 
 
-def generator(excel_file, flow_sheets, save_dir):
-    excel = xlrd.open_workbook(excel_file)
+def check_schedule_trigger(excel):
+    # 1 打开scheduler页，获取所有project-->flow_name的map
+    ss = excel.sheet_by_name('scheduler')
+    st = excel.sheet_by_name('trigger')
+    s_map = {}
+    for i in range(1, ss.nrows):
+        rv = ss.row_values(i)
+        if rv[3] == 1 and rv[1] != '' and rv[0] != '':
+            k = rv[0] + rv[1]
+            v = chr(66) + str(i + 1)
+            if s_map.get(k):
+                raise Exception('Duplicate workflow in scheduler: %s value: %s' % (v, rv[0] + '-' + rv[1]))
+            else:
+                s_map[k] = v
+    # 2 打开trigger页，获取所有project-->flow_name的map，check dep_name是否冲突，将冲突的部分坐标返回
+    for i in range(1, st.nrows):
+        rv = st.row_values(i)
+        if rv[0] == 1 and rv[1] != '' and rv[2] != '':
+            k1 = rv[1] + rv[2]
+            v1 = chr(67) + str(i + 1)
+            try:
+                if s_map[k1]:
+                    raise Exception('scheduler-->%s conflict with trigger-->%s  project flow: %s' % (s_map[k1], v1, k1))
+            except KeyError:
+                pass
+
+
+'''
+add trigger for some workflow by configuration in sheet named "trigger" 
+'''
+
+
+def add_triggers(flows, excel, project):
+    trigger_sheet_name = 'trigger'
+    ts = excel.sheet_by_name(trigger_sheet_name)
+    # flows.get('trigger', collections.OrderedDict())
+    project_name = 'default'
+    dep_list = {}
+    for i in range(1, ts.nrows):
+        rv = ts.row_values(i)
+        enable = rv[0]
+        project_name = rv[1]
+        flow_name = rv[2]
+        if enable == 1 and project_name == project and flow_name != "":
+            flow = flows.get(flow_name)
+            if not flow:
+                pass
+            trigger = flow.get('trigger', collections.OrderedDict())
+            max_min = 1440
+            if rv[4]:
+                max_min = int(rv[4])
+            trigger['maxWaitMins'] = max_min
+
+            schedule_1 = trigger.get('schedule', collections.OrderedDict())
+            schedule_1['type'] = 'cron'
+            cron = "0 10 * * * ?"
+            if rv[3]:
+                cron = rv[3]
+            schedule_1['value'] = cron
+            trigger['schedule'] = schedule_1
+
+            deps = trigger.get('triggerDependencies', [])
+            dep = collections.OrderedDict()
+            params = collections.OrderedDict()
+            k = project_name + flow_name
+
+            dl = dep_list.get(k, [])
+            if dl.__contains__(rv[5]):
+                raise Exception("duplicate dependency:\n%d %s\n%d %s" % (i, ts.row_values(i - 1), i+1, rv))
+            else:
+                dl.append(rv[5])
+                dep_list[k] = dl
+            dep['name'] = rv[5]
+            dep['type'] = rv[6]
+            params['match'] = rv[7]
+            params['topic'] = rv[8]
+            dep['params'] = params
+            deps.append(dep)
+
+            trigger['triggerDependencies'] = deps
+            flow['trigger'] = trigger
+            flows[flow_name] = flow
+
+    return flows
+
+
+'''
+generate all flows file in base directory 
+'''
+
+
+def generator(excel, flow_sheets, save_dir):
+    # check trigger and schedule conflict
+    check_schedule_trigger(excel)
     for project in flow_sheets:
         flows = parse_flows(excel, project)
+        flows = add_triggers(flows, excel, project)
         project_dir = save_dir + os.sep + project
         handle_dir(project_dir, project)
         for f in flows:
@@ -228,16 +321,15 @@ create enabled project on Azkaban Server
 '''
 
 
-def create_project(excel_file, url, session):
-    excel = xlrd.open_workbook(excel_file)
+def create_project(excel, url, session):
     ps = excel.sheet_by_name('projects')
     p_ids = {}
     for r in range(1, ps.nrows):
-        p_name = ps.cell_value(r, 0)
-        enable = ps.cell_value(r, 2)
+        p_name = ps.cell_value(r, 1)
+        enable = ps.cell_value(r, 0)
         if (not enable) or p_name == '':
             continue
-        p_desc = ps.cell_value(r, 1)
+        p_desc = ps.cell_value(r, 2)
         if p_desc == '':
             p_desc = p_name
         params = (
@@ -254,7 +346,6 @@ def create_project(excel_file, url, session):
             print("%s-->%s :%s" % (p_name, p_desc, response.json()))
         p_ids[p_name] = fetch_projects_id(url, session, p_name)
     print("============create project Successfully!============")
-    print('project_name -> project_id:', p_ids)
     return p_ids
 
 
@@ -339,24 +430,51 @@ def fetch_schedule_id(url, session, project_id, flow_name):
     return None
 
 
+'''
+set a sla for a scheduled flow ,which configuration at scheduler sheet,kill or send email is required
+'''
+
+
+def set_sla(url, session, schedule_id, sla_emails, s):
+    email_switch, kill_switch = 'false', 'false'
+    if s[3].value:
+        email_switch = 'true'
+    if s[4].value:
+        kill_switch = 'true'
+    sets = s[0].value + ',' + s[1].value + ',' + s[2].value + ',' + email_switch + ',' + kill_switch
+    params = {
+        'ajax': 'setSla',
+        'slaEmails': sla_emails,
+        'scheduleId': schedule_id,
+        'settings[0]': sets
+    }
+    response = session.post(url + 'schedule', params=params, headers=HEADERS, verify=False)
+    if response:
+        print("set sla for scheduleId ", response.json(), schedule_id)
+
+
 '''
 Which flow was scheduled must be enable in scheduler and the owner of the flow must be enable also.
 '''
 
 
-def schedule(excel_file, url, session, maps):
-    excel = xlrd.open_workbook(excel_file)
+def schedule(excel, url, session, maps):
     schedule_sheet = excel.sheet_by_name('scheduler')
     for row in range(1, schedule_sheet.nrows):
-        project_name = schedule_sheet.cell_value(row, 0).strip()
-        flow_name = schedule_sheet.cell_value(row, 1).strip()
-        cron = schedule_sheet.cell_value(row, 2).strip()
-        enable = schedule_sheet.cell_value(row, 3)
+        project_name = schedule_sheet.cell_value(row, 1).strip()
+        flow_name = schedule_sheet.cell_value(row, 2).strip()
+        cron = schedule_sheet.cell_value(row, 3).strip()
+        enable = schedule_sheet.cell_value(row, 0)
         if project_name == '' or flow_name == '':
             continue
         if project_name and flow_name:
             if cron != '' and enable and maps.get(project_name, None):
-                schedule_flow(url, session, project_name, flow_name, cron)
+                sla_enable = schedule_sheet.cell_value(row, 7)
+                schedule_id = schedule_flow(url, session, project_name, flow_name, cron)
+                if sla_enable:
+                    emails = schedule_sheet.cell_value(row, 8)
+                    sla_setting = schedule_sheet.row_slice(row, 9, 14)
+                    set_sla(url, session, schedule_id, emails, sla_setting)
             else:
                 if maps.get(project_name, None):
                     schedule_id = fetch_schedule_id(url, session, maps[project_name], flow_name)
@@ -373,7 +491,6 @@ def upload_project(url, session, base_dir, project):
     from requests_toolbelt.multipart.encoder import MultipartEncoder
     zip_file_name = project + '.zip'
     zip_path = base_dir + os.sep + zip_file_name
-
     upload_data = MultipartEncoder(
         fields={
             'ajax': 'upload',
@@ -405,8 +522,7 @@ exclude project which enable column is False and default_sheet
 '''
 
 
-def get_valid_projects(excel_file):
-    xl = xlrd.open_workbook(excel_file)
+def get_valid_projects(xl):
     all_sheets = xl.sheet_names()
     for default_sheet in default_sheets:
         all_sheets.remove(default_sheet)
@@ -419,14 +535,16 @@ def get_valid_projects(excel_file):
 
 
 def usage():
-    print('''Usage: azkaban_helper [-h|-g|-c|-z|-u|-o|-s] [--help|--generate|--create|--zip|--upload|--output dirname|--schedule] excel_file
-             -g|--generate generate flows of project,no zip and other operators
-             -c|--create   create projects at azkaban
-             -u|--upload   upload zip file to azkaban server, it will attempt create project before upload
-             -s|--schedule if you just only want to modify a flows scheduler but not modify flow's content,use it
-             excel_file    flow's configuration file must be specified     
+    print('''Usage: azkaban_helper [options] excel_file
+    
+    Options:
+             -g|--generate     generate flows of project,no zip and other operators
+             -c|--create       create projects at azkaban
+             -u|--upload       upload zip file to azkaban server, it will attempt create project before upload
+             -s|--schedule     if you just only want to modify a flows scheduler but not modify flow's content,use it
+                excel_file     flow's configuration file must be specified     
 
-             default:  execute all the steps above.
+             default behavior is executes all the steps above.
           ''')
 
 
@@ -436,14 +554,6 @@ def handle_args():
     try:
         opts, args = getopt.getopt(sys.argv[1:], 'hgcus',
                                    ['help', 'generate', 'create', 'upload', 'schedule'])
-        excel_file = args[0]
-        if not excel_file:
-            usage()
-            print("Excel file is Required!!!")
-            sys.exit(-1)
-        if not os.path.exists(excel_file):
-            print(excel_file, 'is not exists')
-            sys.exit(-2)
         for o, a in opts:
             if o in ('-h', '--help'):
                 usage()
@@ -456,82 +566,105 @@ def handle_args():
                 upload_only = True
             if o in ('-s', '--schedule'):
                 schedule_only = True
+        if not len(args):
+            usage()
+            print("ERROR: Excel file is Required!!!")
+            sys.exit(-1)
+        excel_file = args[0]
+        if not os.path.exists(excel_file):
+            print(excel_file, 'is not exists')
+            sys.exit(-2)
     except getopt.GetoptError:
         usage()
     print(
         '========================\n'
-        'generate_only=%s\ncreate_only  =%s\nupload_only  =%s\nschedule_only=%s\nexcel_file   =%s'
+        ' generate_only=%s\n create_only  =%s\n upload_only  =%s\n schedule_only=%s\n excel_file   =%s'
         '\n========================'
         % (generate_only, create_only, upload_only, schedule_only, excel_file))
+
     return generate_only, create_only, upload_only, schedule_only, excel_file
 
 
+def get_urls_info(excel):
+    url_config = excel.sheet_by_name('config')
+    urls = []
+    for i in range(1, url_config.nrows):
+        u_conf = get_login_config(url_config, i)
+        if u_conf:
+            urls.append(u_conf)
+    return urls
+
+
 '''
 from config sheet read azkaban_url,password,username,base_dir
 '''
 
 
-def get_login_config(excel_file):
-    excel = xlrd.open_workbook(excel_file)
-    config_sheet = excel.sheet_by_name('config')
-    url = config_sheet.cell_value(1, 0).strip()
-    if not url.endswith('/'):
-        url = url + '/'
-    username = config_sheet.cell_value(1, 1).strip()
-    password = config_sheet.cell_value(1, 2).strip()
-    base_dir = config_sheet.cell_value(1, 3).strip()
+def get_login_config(config_sheet, row_index):
+    # only select enable column values is True
+    enable = config_sheet.cell_value(row_index, 0)
+    if enable != 0:
+        url = config_sheet.cell_value(row_index, 1).strip()
+        if not url.endswith('/'):
+            url = url + '/'
+        username = config_sheet.cell_value(row_index, 2).strip()
+        password = config_sheet.cell_value(row_index, 3).strip()
+        base_dir = config_sheet.cell_value(row_index, 4).strip()
 
-    if not base_dir:
-        base_dir = os.getcwd()
+        if not base_dir:
+            base_dir = os.getcwd()
+        else:
+            if base_dir.endswith(os.sep):
+                base_dir = base_dir[:-1]
+        if not os.path.isdir(base_dir):
+            print(base_dir, 'it\'s not exists,auto create it')
+            os.mkdir(base_dir)
+        return url, username, password, base_dir
     else:
-        if base_dir.endswith(os.sep):
-            base_dir = base_dir[:-1]
-    if not os.path.isdir(base_dir):
-        print(base_dir, 'is not exists,auto create it')
-        os.mkdir(base_dir)
-    print('Azkaban Connection Info: url="%s", username=%s,password=%s' % (url, username, password))
-    return url, username, password, base_dir
+        return None
 
 
 def main():
     requests.packages.urllib3.disable_warnings()
     # handle args
     generate_only, create_only, upload_only, schedule_only, excel_file = handle_args()
+    excel = xlrd.open_workbook(excel_file)
     # get valid sheets
-    valid_sheets = get_valid_projects(excel_file)
-    url, username, password, save_dir = get_login_config(excel_file)
-    if generate_only:
-        generator(excel_file, valid_sheets, save_dir)
-        make_zip(valid_sheets, save_dir)
-        sys.exit()
-    if create_only:
-        session = login(url, username, password)
-        create_project(excel_file, url, session)
-        session.close()
-        sys.exit()
-    if upload_only:
-        session = login(url, username, password)
-        run_upload(url, session, valid_sheets, save_dir)
-        session.close()
-        sys.exit()
-    if schedule_only:
-        session = login(url, username, password)
-        pro_map = create_project(excel_file, url, session)
-        schedule(excel_file, url, session, pro_map)
-        session.close()
-        sys.exit()
-    generator(excel_file, valid_sheets, save_dir)
+    valid_sheets = get_valid_projects(excel)
+    web_configs = get_urls_info(excel)
+    generator(excel, valid_sheets, web_configs[0][3])
     print("============generator projects Successfully!============")
-    make_zip(valid_sheets, save_dir)
+    make_zip(valid_sheets, web_configs[0][3])
     print("============compress projects Successfully!============")
-    # The steps below is needed connection to Azkaban Server
-    session = login(url, username, password)
-    pro_map = create_project(excel_file, url, session)
-    run_upload(url, session, valid_sheets, save_dir)
-    print("============upload projects Successfully!============")
-    schedule(excel_file, url, session, pro_map)
-    print("============schedule flows Successfully!============")
-    session.close()
+    for w in web_configs:
+        url, username, password, save_dir = w[0], w[1], w[2], w[3]
+        if generate_only:
+            sys.exit()
+        if create_only:
+            session = login(url, username, password)
+            create_project(excel, url, session)
+            session.close()
+            sys.exit()
+        if upload_only:
+            session = login(url, username, password)
+            run_upload(url, session, valid_sheets, save_dir)
+            session.close()
+            sys.exit()
+        if schedule_only:
+            session = login(url, username, password)
+            pro_map = create_project(excel, url, session)
+            schedule(excel, url, session, pro_map)
+            session.close()
+            sys.exit()
+
+        # The steps below is needed connection to Azkaban Server
+        session = login(url, username, password)
+        pro_map = create_project(excel, url, session)
+        run_upload(url, session, valid_sheets, save_dir)
+        print("============upload projects Successfully!============")
+        schedule(excel, url, session, pro_map)
+        print("============schedule flows Successfully!============")
+        session.close()
 
 
 if __name__ == '__main__':
